import dask.array as da
from dask.array import Array
from skimage import img_as_ubyte
from skimage.color import rgb2gray

def rgb_to_grey(image: Array, as_ubyte: bool = False) -> Array:
    assert image.shape[-1] == 3, "Input image must be RGB"
    assert image.dtype == "uint8", "Input image must be uint8"
    assert image.ndim == 3, "Input image must be 3D"
    image = da.map_blocks(rgb2gray, image, drop_axis=2, dtype="uint8")
    if as_ubyte:
        image = da.map_blocks(img_as_ubyte, image)
    return image

def rgb_to_gray(image: Array, as_ubyte: bool = False) -> Array:
    return rgb_to_grey(image, as_ubyte)


import dask.array as da
from dask.array import Array
from scematk.colour import rgb_to_grey
from skimage import img_as_ubyte

def grey_to_od(image: Array, clip = False) -> Array:
    assert image.ndim == 2, "Input image must be 2D"
    assert image.dtype in ["float32", "float64", "uint8"], "Input image must be float32, float64 or uint8"
    if image.dtype == "uint8":
        image = image / 255
    od_img = -da.log10(image)
    if clip:
        od_img = da.clip(od_img, 0, 1)
    return od_img

def gray_to_od(image: Array, clip = False) -> Array:
    return grey_to_od(image, clip)

def od_to_grey(image: Array, as_ubyte = False) -> Array:
    assert image.ndim == 2, "Input image must be 2D"
    assert image.dtype in ["float32", "float64"], "Input image must be float32 or float64"
    grey_img = da.exp(-image)
    if as_ubyte:
        grey_img = da.map_blocks(img_as_ubyte, grey_img)
    return grey_img

def od_to_gray(image: Array, as_ubyte = False) -> Array:
    return od_to_grey(image, as_ubyte)

def rgb_to_od(image: Array, clip = False) -> Array:
    image = rgb_to_grey(image)
    return grey_to_od(image, clip)


import dask.array as da
from scematk.colour import rgb_to_grey
from skimage.color import rgb2gray

def test_rgb_to_grey():
    rimg = da.random.randint(0, 255, size=(500, 500, 3), chunks=(50, 50, 3))
    diff = rgb_to_grey(rimg).compute() == rgb2gray(rimg.compute())
    assert diff.all()

def test_int_rgb_to_grey():
    rimg = da.random.randint(0, 255, size=(500, 500, 3), chunks=(50, 50, 3), dtype='int')
    assert isinstance(rgb_to_grey(rimg), da.Array)



from .grey_transform import rgb_to_gray, rgb_to_grey
from .od_transform import grey_to_od, od_to_grey, gray_to_od, od_to_gray, rgb_to_od

__all__ = [
    "gray_to_od",
    "grey_to_od",
    "od_to_gray",
    "od_to_grey",
    "rgb_to_gray",
    "rgb_to_grey",
    "rgb_to_od"
]